@@ -258,21 +258,28 @@ things:
 Here the data are points evently spaced around a unit circle. Along with these
 points we plot a unit circle as a parametric equation.
 
-** Interactivity
+** Plot persistence and blocking
 
-The graphical backends of Gnuplot are interactive, allowing the user to pan,
-zoom, rotate and measure the data in the plot window. See the Gnuplot
-documentation for details about how to do this. Some terminals (such as wxt) are
-persistently interactive, and the rest of this section does not apply to them.
-Other terminals (such as x11) have the downside described here.
+As currently written, gnuplotlib does NOT block and the plot windows do NOT
+persist. I.e.
 
-When using an affected terminal, interactivity is only possible if the gnuplot
-process is running. As long as the python program calling gnuplotlib is running,
-the plots are interactive, but once it exits, the child gnuplot process will
-exit also. This will keep the plot windows up, but the interactivity will be
-lost. So if the python program makes a plot and exits, the plot will NOT be
-interactive.
+- the 'plot()' functions return immediately, and the user interacts with the
+  plot WHILE THE REST OF THE PYTHON PROGRAM IS RUNNING
 
+- when the python program exits, the gnuplot process and any visible plots go
+  away
+
+So if you want to write a program that just shows a plot and does nothing else,
+you MUST put some sort of blocking call after the 'plot()' and before the python
+program exits. Something as simple as the below works:
+
+    gp.plot( ... )
+    import time
+    time.sleep(100000)
+
+The plot will then stay up until the python process is interrupted, with C-c for
+instance. This is an ugly hack, and I have ideas about improving it, but this
+hasn't been a priority so far.
 
 * OPTIONS
 
@@ -708,7 +715,7 @@ import numpy as np
 import numpysane as nps
 
 # setup.py assumes the version is a simple string in '' quotes
-__version__ = '0.18'
+__version__ = '0.20'
 
 # note that 'with' is both a known plot and curve option
 knownPlotOptions = frozenset(('dump', 'ascii', 'log', 'notest',
@@ -732,40 +739,31 @@ testdataunit_ascii = 10
 
 
 
-
-def _runSubprocess(cmd, input):
-    r'''Runs a given command feeding the given input to stdin. Waits for the command
-    to terminate, and returns the stdout,stderr tuple we get back
-
-    '''
-
-    process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    out, err = process.communicate(input=input)
-    retcode = process.poll()
-    if retcode:
-        raise subprocess.CalledProcessError(retcode, cmd[0], output=out + err)
-    return out,err
-
-
 def _getGnuplotFeatures():
 
-    # Be careful talking to gnuplot here. If you use a tty then gnuplot
-    # messes with the tty settings where it should NOT. For example it turns
-    # on the local echo. So make sure to not use a tty
+    # Be careful talking to gnuplot here. If you use a tty then gnuplot messes
+    # with the tty settings where it should NOT. For example it turns on the
+    # local echo. So make sure to not use a tty. Furthermore, I turn off the
+    # DISPLAY. I'm not actually plotting anything, so a DISPLAY can try to
+    # X-forward and be really slow pointlessly
 
     # first, I run 'gnuplot --help' to extract all the cmdline options as features
-    helpstring = subprocess.check_output(['gnuplot', '--help'], stderr=subprocess.STDOUT)
+    helpstring = subprocess.check_output(['gnuplot', '--help'],
+                                         stderr=subprocess.STDOUT,
+                                         env={'DISPLAY': ''})
     features = set( re.findall(r'--([a-zA-Z0-9_]+)', helpstring) )
 
 
     # then I try to set a square aspect ratio for 3D to see if it works
-    out,err = _runSubprocess(['gnuplot'],
-                             """set view equal
-exit
-""")
+    try:
+        out = subprocess.check_output(('gnuplot', '-e', "set view equal"),
+                                      stderr=subprocess.STDOUT,
+                                      env={'DISPLAY': ''})
+    except:
+        out = 'error!'
 
     # no output if works; some output if error
-    if( not ( out or err ) ):
+    if len(out) == 0:
         features.add('equal_3d')
 
     return frozenset(features)
@@ -833,10 +831,12 @@ class gnuplotlib:
         if not self.plotOptions.get('dump'):
 
             cmd = ['gnuplot']
-            if 'persist' in features:
-                cmd += ['--persist']
 
-            self.fdDupSTDOUT = os.dup(sys.stdout.fileno())
+            try:
+                self.fdDupSTDOUT = os.dup(sys.stdout.fileno())
+            except:
+                self.fdDupSTDOUT = None
+
             self.gnuplotProcess = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
         # save the default terminal
@@ -886,6 +886,12 @@ class gnuplotlib:
         # set the plot bounds
         for axis in ('x', 'y', 'y2', 'z', 'cb'):
 
+            # set the curve labels
+            if not axis == 'cb':
+                if axis + 'label' in self.plotOptions:
+                    cmds.append('set {axis}label "{label}"'.format(axis = axis,
+                                                                   label = self.plotOptions[axis + 'label']))
+
             # I deal with range bounds here. These can be given for the various
             # axes by variables (W-axis here; replace W with x, y, z, etc):
             #
@@ -940,12 +946,6 @@ class gnuplotlib:
                                 '*' if opt_max is None else opt_max,
                                 '' if opt_inv else 'no'))
 
-            # set the curve labels
-            if not axis == 'cb':
-                if axis + 'label' in self.plotOptions:
-                    cmds.append('set {axis}label "{label}"'.format(axis = axis,
-                                                                   label = self.plotOptions[axis + 'label']))
-
 
 
         # set the title
@@ -982,11 +982,11 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
 
             outputfileType = m.group(1)
 
-            terminalOpts = { 'eps': 'postscript solid color enhanced eps',
-                             'ps':  'postscript solid color landscape 10',
-                             'pdf': 'pdf solid color font ",10" size 11in,8.5in',
-                             'png': 'png size 1280,1024',
-                             'svg': 'svg enhanced solid'}
+            terminalOpts = { 'eps': 'postscript noenhanced solid color eps',
+                             'ps':  'postscript noenhanced solid color landscape 10',
+                             'pdf': 'pdf noenhanced solid color font ",10" size 11in,8.5in',
+                             'png': 'png noenhanced size 1280,1024',
+                             'svg': 'svg noenhanced solid'}
 
             self.plotOptions['terminal'] = terminalOpts[outputfileType]
             self.plotOptions['output']   = outputfile
@@ -1020,7 +1020,10 @@ defaults are acceptable, use 'hardcopy' only, otherwise use 'terminal' and
 
             self.gnuplotProcess.wait()
             self.gnuplotProcess = None
-            os.close(self.fdDupSTDOUT)
+
+            if self.fdDupSTDOUT is not None:
+                os.close(self.fdDupSTDOUT)
+                self.fdDupSTDOUT = None
 
 
 
@@ -1389,9 +1392,9 @@ labels with spaces in them
         if 'rgbimage' in self.plotOptions:
             if not os.access     (self.plotOptions['rgbimage'], os.R_OK) or \
                not os.path.isfile(self.plotOptions['rgbimage']):
-                raise GnuplotlibError("Requested image '{}' is not a readable file".format(plotOptions['rgbimage']))
+                raise GnuplotlibError("Requested image '{}' is not a readable file".format(self.plotOptions['rgbimage']))
 
-            basecmd += '"{}" binary filetype=auto flipy with rgbimage, '.format(self.plotOptions['rgbimage'])
+            basecmd += '"{0}" binary filetype=auto flipy with rgbimage title "{0}", '.format(self.plotOptions['rgbimage'])
 
         plotCurveCmds        = []
         plotCurveCmdsMinimal = [] # same as above, but with a single data point per plot only
@@ -1648,6 +1651,9 @@ labels with spaces in them
                 self._safelyWriteToPipe('set output',
                                         'output')
             else:
+                if self.fdDupSTDOUT is None:
+                    raise Exception("I need to plot to STDOUT, but STDOUT wasn't available")
+
                 self.plotOptions['output'] = '/dev/fd/' + str(self.fdDupSTDOUT)
                 self._safelyWriteToPipe('set output "' + self.plotOptions['output'] + '"',
                                         'output')
